﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Xml;
using System.Xml.Linq;
using DynamicwebADIntegration.Entities;

namespace DynamicwebADIntegration
{
    public class XmlFormatter
    {
        /// <summary>
        /// Returns users and groups in xml format for import to Dynamicweb
        /// </summary>
        /// <param name="users"></param>
        /// <param name="groups"></param>
        /// <returns></returns>
        public static XmlDocument GetUsersGroupsXml(List<User> users, List<Group> groups, string importGroupID)
        {
            XmlDocument xDoc = new XmlDocument();
            XmlDeclaration xDec = xDoc.CreateXmlDeclaration("1.0", "UTF-8", "yes");
            xDoc.AppendChild(xDec);

            XmlElement xRoot = xDoc.CreateElement("tables");
            

            XmlElement usersNode = xDoc.CreateElement("table");
            usersNode.SetAttribute("tableName", "AccessUser");

            foreach (User user in users)
            {
                XmlElement item = xDoc.CreateElement("item");
                item.SetAttribute("table", "AccessUser");

                item.AppendChild(GetColumnTag(xDoc, "AccessUserUserName", user.Email));
                item.AppendChild(GetColumnTag(xDoc, "AccessUserName", user.Name));
                item.AppendChild(GetColumnTag(xDoc, "AccessUserFirstName", user.GivenName));
                item.AppendChild(GetColumnTag(xDoc, "AccessUserLastName", user.Surname));
                item.AppendChild(GetColumnTag(xDoc, "AccessUserMiddleName", user.MiddleName));
                item.AppendChild(GetColumnTag(xDoc, "AccessUserEmail", user.Email));
                item.AppendChild(GetColumnTag(xDoc, "AccessUserExternalID", user.EmployeeId));
                item.AppendChild(GetColumnTag(xDoc, "AccessUserPhone", user.VoiceTelephoneNumber));
                item.AppendChild(GetColumnTag(xDoc, "AccessUserActive", user.Enabled.HasValue ? user.Enabled.Value.ToString().ToLower() : "true"));
                item.AppendChild(GetColumnTag(xDoc, "AccessUserGroups", string.Join(",", user.Groups.Select(g => g.Name).ToList())));

                usersNode.AppendChild(item); 
            }
            xRoot.AppendChild(usersNode);

            XmlElement groupsNode = xDoc.CreateElement("table");
            groupsNode.SetAttribute("tableName", "AccessUserGroup");

            foreach (Group group in groups)
            {
                XmlElement item = xDoc.CreateElement("item");
                item.SetAttribute("table", "AccessUserGroup");
                item.AppendChild(GetColumnTag(xDoc, "AccessGroupName", group.Name));
                item.AppendChild(GetColumnTag(xDoc, "AccessGroupGroupName", group.Name));
                item.AppendChild(GetColumnTag(xDoc, "AccessGroupParentGroupName", importGroupID));                
                groupsNode.AppendChild(item);
            }
            xRoot.AppendChild(groupsNode);

            xDoc.AppendChild(xRoot);
            return xDoc;
        }

        private static XmlElement GetColumnTag(XmlDocument xDoc, string name, string value)
        {
            XmlElement column = xDoc.CreateElement("column");
            column.SetAttribute("columnName", name);
            column.AppendChild(xDoc.CreateCDataSection(value));            
            return column;
        }
    }
}